"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from jsonpath import JSONPath
from kombo import errors, models, utils
from kombo._hooks import HookContext
from kombo.types import BaseModel, OptionalNullable, UNSET
from kombo.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Awaitable, Dict, List, Mapping, Optional, Union, cast


class Assessment(BaseSDK):
    def get_packages(
        self,
        *,
        request: Union[
            models.GetAssessmentPackagesRequest,
            models.GetAssessmentPackagesRequestTypedDict,
        ] = models.GetAssessmentPackagesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetAssessmentPackagesPositiveResponse:
        r"""Get packages

        Get all available assessment and background check packages for an integration.

        This is mainly intended for debugging. As you always need to submit the full list of available packages when using [\"set packages\"](/assessment/v1/put-packages), there shouldn't ever be a need to call this endpoint in production.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.GetAssessmentPackagesRequest)
        request = cast(models.GetAssessmentPackagesRequest, request)

        req = self._build_request(
            method="GET",
            path="/assessment/packages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.GetAssessmentPackagesGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="GetAssessmentPackages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.GetAssessmentPackagesPositiveResponse, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_packages_async(
        self,
        *,
        request: Union[
            models.GetAssessmentPackagesRequest,
            models.GetAssessmentPackagesRequestTypedDict,
        ] = models.GetAssessmentPackagesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetAssessmentPackagesPositiveResponse:
        r"""Get packages

        Get all available assessment and background check packages for an integration.

        This is mainly intended for debugging. As you always need to submit the full list of available packages when using [\"set packages\"](/assessment/v1/put-packages), there shouldn't ever be a need to call this endpoint in production.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.GetAssessmentPackagesRequest)
        request = cast(models.GetAssessmentPackagesRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/assessment/packages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.GetAssessmentPackagesGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="GetAssessmentPackages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.GetAssessmentPackagesPositiveResponse, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def set_packages(
        self,
        *,
        packages: Union[
            List[models.PutAssessmentPackagesRequestBodyPackage],
            List[models.PutAssessmentPackagesRequestBodyPackageTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PutAssessmentPackagesPositiveResponse:
        r"""Set packages

        Set packages

        Replaces the list of available assessment and or background check packages.

        Packages that have been previously submitted through this endpoint but aren't included again will be marked as deleted.

        ### Example Request Body

        ```json
        {
        \"packages\": [
        {
        \"id\": \"1001\",
        \"type\": \"SKILLS_TEST\",
        \"name\": \"TypeScript\",
        \"description\": \"TypeScript coding skills assessments\"
        },
        {
        \"id\": \"1002\",
        \"type\": \"VIDEO_INTERVIEW\",
        \"name\": \"Video Interview\",
        \"description\": \"Video interview to assess communication skills\"
        }
        ]
        }
        ```

        :param packages:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutAssessmentPackagesRequestBody(
            packages=utils.get_pydantic_model(
                packages, List[models.PutAssessmentPackagesRequestBodyPackage]
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/assessment/packages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.PutAssessmentPackagesGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.PutAssessmentPackagesRequestBody
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="PutAssessmentPackages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.PutAssessmentPackagesPositiveResponse, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def set_packages_async(
        self,
        *,
        packages: Union[
            List[models.PutAssessmentPackagesRequestBodyPackage],
            List[models.PutAssessmentPackagesRequestBodyPackageTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PutAssessmentPackagesPositiveResponse:
        r"""Set packages

        Set packages

        Replaces the list of available assessment and or background check packages.

        Packages that have been previously submitted through this endpoint but aren't included again will be marked as deleted.

        ### Example Request Body

        ```json
        {
        \"packages\": [
        {
        \"id\": \"1001\",
        \"type\": \"SKILLS_TEST\",
        \"name\": \"TypeScript\",
        \"description\": \"TypeScript coding skills assessments\"
        },
        {
        \"id\": \"1002\",
        \"type\": \"VIDEO_INTERVIEW\",
        \"name\": \"Video Interview\",
        \"description\": \"Video interview to assess communication skills\"
        }
        ]
        }
        ```

        :param packages:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutAssessmentPackagesRequestBody(
            packages=utils.get_pydantic_model(
                packages, List[models.PutAssessmentPackagesRequestBodyPackage]
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/assessment/packages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.PutAssessmentPackagesGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.PutAssessmentPackagesRequestBody
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="PutAssessmentPackages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.PutAssessmentPackagesPositiveResponse, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_open_orders(
        self,
        *,
        cursor: Optional[str] = None,
        page_size: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.GetAssessmentOrdersOpenResponse]:
        r"""Get open orders

        Get all open assessment and background check orders of an integration.

        :param cursor: An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response.
        :param page_size: The number of results to return per page. Maximum is 250.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetAssessmentOrdersOpenRequest(
            cursor=cursor,
            page_size=page_size,
        )

        req = self._build_request(
            method="GET",
            path="/assessment/orders/open",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.GetAssessmentOrdersOpenGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="GetAssessmentOrdersOpen",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.GetAssessmentOrdersOpenResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            next_cursor = JSONPath("$.data.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None

            return self.get_open_orders(
                cursor=next_cursor,
                page_size=page_size,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetAssessmentOrdersOpenResponse(
                result=unmarshal_json_response(
                    models.GetAssessmentOrdersOpenPositiveResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_open_orders_async(
        self,
        *,
        cursor: Optional[str] = None,
        page_size: Optional[int] = 100,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.GetAssessmentOrdersOpenResponse]:
        r"""Get open orders

        Get all open assessment and background check orders of an integration.

        :param cursor: An optional cursor string used for pagination. This can be retrieved from the `next` property of the previous page response.
        :param page_size: The number of results to return per page. Maximum is 250.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetAssessmentOrdersOpenRequest(
            cursor=cursor,
            page_size=page_size,
        )

        req = self._build_request_async(
            method="GET",
            path="/assessment/orders/open",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.GetAssessmentOrdersOpenGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="GetAssessmentOrdersOpen",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        def next_func() -> Awaitable[Optional[models.GetAssessmentOrdersOpenResponse]]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            async def empty_result():
                return None

            next_cursor = JSONPath("$.data.next").parse(body)

            if len(next_cursor) == 0:
                return empty_result()

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return empty_result()

            return self.get_open_orders_async(
                cursor=next_cursor,
                page_size=page_size,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetAssessmentOrdersOpenResponse(
                result=unmarshal_json_response(
                    models.GetAssessmentOrdersOpenPositiveResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update_order_result(
        self,
        *,
        assessment_order_id: str,
        status: models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyStatus,
        result_url: str,
        completed_at: Optional[datetime] = None,
        score: Optional[float] = None,
        max_score: Optional[float] = None,
        attributes: Optional[
            Union[List[models.Attribute], List[models.AttributeTypedDict]]
        ] = None,
        attachments: Optional[
            Union[
                List[
                    models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachment
                ],
                List[
                    models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachmentTypedDict
                ],
            ]
        ] = None,
        remote_fields: Optional[
            Union[
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFields,
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFieldsTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PutAssessmentOrdersAssessmentOrderIDResultPositiveResponse:
        r"""Update order result

        Updates an assessment or a background check order result.

        ### Example Request Body

        ```json
        {
        \"status\": \"COMPLETED\",
        \"score\": 90,
        \"max_score\": 100,
        \"result_url\": \"https://example.com\",
        \"completed_at\": \"2023-04-04T00:00:00.000Z\",
        \"attributes\": [
        {
        \"type\": \"TEXT\",
        \"label\": \"Role fit\",
        \"value\": \"Excellent\"
        },
        {
        \"type\": \"SUB_RESULT\",
        \"id\": \"<YOUR_INTERNAL_ID_OF_THE_TEST>\",
        \"label\": \"Personality test\",
        \"score\": {
        \"value\": 97,
        \"max\": 100
        },
        \"status\": \"COMPLETED\"
        }
        ],
        \"attachments\": [
        {
        \"name\": \"Assessment Report.pdf\",
        \"data\": \"SGkgdGhlcmUsIEtvbWJvIGlzIGN1cnJlbnRseSBoaXJpbmcgZW5naW5lZXJzIHRoYXQgbG92ZSB0byB3b3JrIG9uIGRldmVsb3BlciBwcm9kdWN0cy4=\",
        \"content_type\": \"application/pdf\"
        }
        ]
        }
        ```

        :param assessment_order_id: PUT /assessment/orders/:assessment_order_id/result Parameter
        :param status: Status of the assessment.

            **Please note the `status` can only be updated to a different value if its current value is `OPEN`.**
        :param result_url:
        :param completed_at: YYYY-MM-DDTHH:mm:ss.sssZ

            **Please make sure this value is provided when the `status` is of the type `COMPLETED` or `CANCELLED`.**
        :param score:
        :param max_score:
        :param attributes: An array of additional attributes that you would like to submit as a part of the assessment result.

            - If an ATS only supports writing text attributes, we will transform non `TEXT` attributes into formatted plain text values.
        :param attachments: An array of attachments containing the assessment result.
        :param remote_fields: Additional fields that we will pass through to specific ATS systems.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutAssessmentOrdersAssessmentOrderIDResultRequest(
            assessment_order_id=assessment_order_id,
            body=models.PutAssessmentOrdersAssessmentOrderIDResultRequestBody(
                status=status,
                result_url=result_url,
                completed_at=completed_at,
                score=score,
                max_score=max_score,
                attributes=utils.get_pydantic_model(
                    attributes, Optional[List[models.Attribute]]
                ),
                attachments=utils.get_pydantic_model(
                    attachments,
                    Optional[
                        List[
                            models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachment
                        ]
                    ],
                ),
                remote_fields=utils.get_pydantic_model(
                    remote_fields,
                    Optional[
                        models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFields
                    ],
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/assessment/orders/{assessment_order_id}/result",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.PutAssessmentOrdersAssessmentOrderIDResultGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBody,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="PutAssessmentOrdersAssessmentOrderIdResult",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.PutAssessmentOrdersAssessmentOrderIDResultPositiveResponse,
                http_res,
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_order_result_async(
        self,
        *,
        assessment_order_id: str,
        status: models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyStatus,
        result_url: str,
        completed_at: Optional[datetime] = None,
        score: Optional[float] = None,
        max_score: Optional[float] = None,
        attributes: Optional[
            Union[List[models.Attribute], List[models.AttributeTypedDict]]
        ] = None,
        attachments: Optional[
            Union[
                List[
                    models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachment
                ],
                List[
                    models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachmentTypedDict
                ],
            ]
        ] = None,
        remote_fields: Optional[
            Union[
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFields,
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFieldsTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PutAssessmentOrdersAssessmentOrderIDResultPositiveResponse:
        r"""Update order result

        Updates an assessment or a background check order result.

        ### Example Request Body

        ```json
        {
        \"status\": \"COMPLETED\",
        \"score\": 90,
        \"max_score\": 100,
        \"result_url\": \"https://example.com\",
        \"completed_at\": \"2023-04-04T00:00:00.000Z\",
        \"attributes\": [
        {
        \"type\": \"TEXT\",
        \"label\": \"Role fit\",
        \"value\": \"Excellent\"
        },
        {
        \"type\": \"SUB_RESULT\",
        \"id\": \"<YOUR_INTERNAL_ID_OF_THE_TEST>\",
        \"label\": \"Personality test\",
        \"score\": {
        \"value\": 97,
        \"max\": 100
        },
        \"status\": \"COMPLETED\"
        }
        ],
        \"attachments\": [
        {
        \"name\": \"Assessment Report.pdf\",
        \"data\": \"SGkgdGhlcmUsIEtvbWJvIGlzIGN1cnJlbnRseSBoaXJpbmcgZW5naW5lZXJzIHRoYXQgbG92ZSB0byB3b3JrIG9uIGRldmVsb3BlciBwcm9kdWN0cy4=\",
        \"content_type\": \"application/pdf\"
        }
        ]
        }
        ```

        :param assessment_order_id: PUT /assessment/orders/:assessment_order_id/result Parameter
        :param status: Status of the assessment.

            **Please note the `status` can only be updated to a different value if its current value is `OPEN`.**
        :param result_url:
        :param completed_at: YYYY-MM-DDTHH:mm:ss.sssZ

            **Please make sure this value is provided when the `status` is of the type `COMPLETED` or `CANCELLED`.**
        :param score:
        :param max_score:
        :param attributes: An array of additional attributes that you would like to submit as a part of the assessment result.

            - If an ATS only supports writing text attributes, we will transform non `TEXT` attributes into formatted plain text values.
        :param attachments: An array of attachments containing the assessment result.
        :param remote_fields: Additional fields that we will pass through to specific ATS systems.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutAssessmentOrdersAssessmentOrderIDResultRequest(
            assessment_order_id=assessment_order_id,
            body=models.PutAssessmentOrdersAssessmentOrderIDResultRequestBody(
                status=status,
                result_url=result_url,
                completed_at=completed_at,
                score=score,
                max_score=max_score,
                attributes=utils.get_pydantic_model(
                    attributes, Optional[List[models.Attribute]]
                ),
                attachments=utils.get_pydantic_model(
                    attachments,
                    Optional[
                        List[
                            models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyAttachment
                        ]
                    ],
                ),
                remote_fields=utils.get_pydantic_model(
                    remote_fields,
                    Optional[
                        models.PutAssessmentOrdersAssessmentOrderIDResultRequestBodyRemoteFields
                    ],
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/assessment/orders/{assessment_order_id}/result",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.PutAssessmentOrdersAssessmentOrderIDResultGlobals(
                integration_id=self.sdk_configuration.globals.integration_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.PutAssessmentOrdersAssessmentOrderIDResultRequestBody,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="PutAssessmentOrdersAssessmentOrderIdResult",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["default"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.PutAssessmentOrdersAssessmentOrderIDResultPositiveResponse,
                http_res,
            )
        if utils.match_response(http_res, "default", "application/json"):
            response_data = unmarshal_json_response(errors.KomboAtsErrorData, http_res)
            raise errors.KomboAtsError(response_data, http_res)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PostAtsApplicationsApplicationIDRejectRequestBodyPostHeadersTypedDict(TypedDict):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: NotRequired[Nullable[str]]
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""


class PostAtsApplicationsApplicationIDRejectRequestBodyPostHeaders(BaseModel):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: Annotated[
        OptionalNullable[str], pydantic.Field(alias="On-Behalf-Of")
    ] = UNSET
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["On-Behalf-Of"])
        nullable_fields = set(["On-Behalf-Of"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDRejectRequestBodyGreenhouseTypedDict(TypedDict):
    r"""Fields specific to Greenhouse."""

    post_headers: NotRequired[
        PostAtsApplicationsApplicationIDRejectRequestBodyPostHeadersTypedDict
    ]
    r"""Headers we will pass with `POST` requests to Greenhouse."""


class PostAtsApplicationsApplicationIDRejectRequestBodyGreenhouse(BaseModel):
    r"""Fields specific to Greenhouse."""

    post_headers: Optional[
        PostAtsApplicationsApplicationIDRejectRequestBodyPostHeaders
    ] = None
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["post_headers"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDRejectRequestBodyTeamtailorTypedDict(TypedDict):
    r"""Fields specific to Teamtailor."""

    user_id: NotRequired[str]
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action. If not provided, the first admin user will be used."""


class PostAtsApplicationsApplicationIDRejectRequestBodyTeamtailor(BaseModel):
    r"""Fields specific to Teamtailor."""

    user_id: Optional[str] = None
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action. If not provided, the first admin user will be used."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["user_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDRejectRequestBodyWorkableTypedDict(TypedDict):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: NotRequired[str]
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""


class PostAtsApplicationsApplicationIDRejectRequestBodyWorkable(BaseModel):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: Optional[str] = None
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["on_behalf_of_user_remote_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDRejectRequestBodyRemoteFieldsTypedDict(TypedDict):
    r"""Additional fields that we will pass through to specific ATS systems."""

    greenhouse: NotRequired[
        PostAtsApplicationsApplicationIDRejectRequestBodyGreenhouseTypedDict
    ]
    r"""Fields specific to Greenhouse."""
    teamtailor: NotRequired[
        PostAtsApplicationsApplicationIDRejectRequestBodyTeamtailorTypedDict
    ]
    r"""Fields specific to Teamtailor."""
    workable: NotRequired[
        PostAtsApplicationsApplicationIDRejectRequestBodyWorkableTypedDict
    ]
    r"""Workable specific remote fields for ATS actions."""


class PostAtsApplicationsApplicationIDRejectRequestBodyRemoteFields(BaseModel):
    r"""Additional fields that we will pass through to specific ATS systems."""

    greenhouse: Optional[
        PostAtsApplicationsApplicationIDRejectRequestBodyGreenhouse
    ] = None
    r"""Fields specific to Greenhouse."""

    teamtailor: Optional[
        PostAtsApplicationsApplicationIDRejectRequestBodyTeamtailor
    ] = None
    r"""Fields specific to Teamtailor."""

    workable: Optional[PostAtsApplicationsApplicationIDRejectRequestBodyWorkable] = None
    r"""Workable specific remote fields for ATS actions."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["greenhouse", "teamtailor", "workable"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsApplicationsApplicationIDRejectRequestBodyTypedDict(TypedDict):
    rejection_reason_id: str
    r"""The Kombo ID of the rejection reason."""
    note: NotRequired[str]
    r"""A optional free text rejection note. Passed through if possible."""
    remote_fields: NotRequired[
        PostAtsApplicationsApplicationIDRejectRequestBodyRemoteFieldsTypedDict
    ]
    r"""Additional fields that we will pass through to specific ATS systems."""


class PostAtsApplicationsApplicationIDRejectRequestBody(BaseModel):
    rejection_reason_id: str
    r"""The Kombo ID of the rejection reason."""

    note: Optional[str] = None
    r"""A optional free text rejection note. Passed through if possible."""

    remote_fields: Optional[
        PostAtsApplicationsApplicationIDRejectRequestBodyRemoteFields
    ] = None
    r"""Additional fields that we will pass through to specific ATS systems."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["note", "remote_fields"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PostAtsApplicationsApplicationIDRejectRequestBodyPostHeaders.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from kombo.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import (
    Annotated,
    NotRequired,
    TypeAliasType,
    TypedDict,
    deprecated,
)


PostAtsCandidatesRequestBodyAdditionalEmailAddressType = Literal[
    "PERSONAL",
    "WORK",
    "OTHER",
]


class PostAtsCandidatesRequestBodyAdditionalEmailAddressTypedDict(TypedDict):
    type: PostAtsCandidatesRequestBodyAdditionalEmailAddressType
    email_address: str


class PostAtsCandidatesRequestBodyAdditionalEmailAddress(BaseModel):
    type: PostAtsCandidatesRequestBodyAdditionalEmailAddressType

    email_address: str


PostAtsCandidatesRequestBodyAdditionalPhoneNumberType = Literal[
    "PERSONAL",
    "WORK",
    "OTHER",
]


class PostAtsCandidatesRequestBodyAdditionalPhoneNumberTypedDict(TypedDict):
    type: PostAtsCandidatesRequestBodyAdditionalPhoneNumberType
    phone_number: str


class PostAtsCandidatesRequestBodyAdditionalPhoneNumber(BaseModel):
    type: PostAtsCandidatesRequestBodyAdditionalPhoneNumberType

    phone_number: str


class PostAtsCandidatesRequestBodyLocationTypedDict(TypedDict):
    r"""The location of the candidate."""

    country: str
    r"""The uppercase two-letter ISO country (e.g., `DE`). For systems that use codes in formats other than `ISO 3166-1 alpha-2`, Kombo transforms the ISO Codes to the appropriate value."""
    city: NotRequired[str]
    state: NotRequired[str]
    street_1: NotRequired[str]
    zip_code: NotRequired[str]


class PostAtsCandidatesRequestBodyLocation(BaseModel):
    r"""The location of the candidate."""

    country: str
    r"""The uppercase two-letter ISO country (e.g., `DE`). For systems that use codes in formats other than `ISO 3166-1 alpha-2`, Kombo transforms the ISO Codes to the appropriate value."""

    city: Optional[str] = None

    state: Optional[str] = None

    street_1: Optional[str] = None

    zip_code: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["city", "state", "street_1", "zip_code"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


PostAtsCandidatesRequestBodyGender = Literal[
    "MALE",
    "FEMALE",
    "OTHER",
]
r"""The gender of the candidate. Must be one of `MALE`, `FEMALE`, or `OTHER`."""


PostAtsCandidatesRequestBodyPeriod = Literal[
    "MONTH",
    "YEAR",
]
r"""The period of the salary expectations. Must be one of `MONTH` or `YEAR`."""


class PostAtsCandidatesRequestBodySalaryExpectationsTypedDict(TypedDict):
    r"""The salary expectations of the applicant. We will automatically convert the amount to a format that is suitable for the ATS you are using. For example, if you are using monthly salary expectations, we will convert the amount to a yearly salary if the ATS expects yearly salary expectations."""

    period: PostAtsCandidatesRequestBodyPeriod
    r"""The period of the salary expectations. Must be one of `MONTH` or `YEAR`."""
    amount: float
    r"""The amount of the salary expectations."""


class PostAtsCandidatesRequestBodySalaryExpectations(BaseModel):
    r"""The salary expectations of the applicant. We will automatically convert the amount to a format that is suitable for the ATS you are using. For example, if you are using monthly salary expectations, we will convert the amount to a yearly salary if the ATS expects yearly salary expectations."""

    period: PostAtsCandidatesRequestBodyPeriod
    r"""The period of the salary expectations. Must be one of `MONTH` or `YEAR`."""

    amount: float
    r"""The amount of the salary expectations."""


class PostAtsCandidatesRequestBodySocialLinkTypedDict(TypedDict):
    url: str


class PostAtsCandidatesRequestBodySocialLink(BaseModel):
    url: str


class PostAtsCandidatesRequestBodyCandidateTypedDict(TypedDict):
    first_name: str
    r"""The first name of the candidate."""
    last_name: str
    r"""The last name of the candidate."""
    email_address: str
    r"""The primary email address this application will be created with."""
    additional_email_addresses: NotRequired[
        List[PostAtsCandidatesRequestBodyAdditionalEmailAddressTypedDict]
    ]
    company: NotRequired[str]
    r"""The company where the candidate is currently working."""
    title: NotRequired[str]
    r"""The current job title of the candidate."""
    phone_number: NotRequired[str]
    r"""The phone number of the candidate."""
    additional_phone_numbers: NotRequired[
        List[PostAtsCandidatesRequestBodyAdditionalPhoneNumberTypedDict]
    ]
    location: NotRequired[PostAtsCandidatesRequestBodyLocationTypedDict]
    r"""The location of the candidate."""
    gender: NotRequired[PostAtsCandidatesRequestBodyGender]
    r"""The gender of the candidate. Must be one of `MALE`, `FEMALE`, or `OTHER`."""
    availability_date: NotRequired[datetime]
    r"""The date the candidate is available to start working.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    salary_expectations: NotRequired[
        PostAtsCandidatesRequestBodySalaryExpectationsTypedDict
    ]
    r"""The salary expectations of the applicant. We will automatically convert the amount to a format that is suitable for the ATS you are using. For example, if you are using monthly salary expectations, we will convert the amount to a yearly salary if the ATS expects yearly salary expectations."""
    social_links: NotRequired[List[PostAtsCandidatesRequestBodySocialLinkTypedDict]]
    r"""A list of social media links of the candidate. The links must be valid URLs."""


class PostAtsCandidatesRequestBodyCandidate(BaseModel):
    first_name: str
    r"""The first name of the candidate."""

    last_name: str
    r"""The last name of the candidate."""

    email_address: str
    r"""The primary email address this application will be created with."""

    additional_email_addresses: Optional[
        List[PostAtsCandidatesRequestBodyAdditionalEmailAddress]
    ] = None

    company: Optional[str] = None
    r"""The company where the candidate is currently working."""

    title: Optional[str] = None
    r"""The current job title of the candidate."""

    phone_number: Optional[str] = None
    r"""The phone number of the candidate."""

    additional_phone_numbers: Optional[
        List[PostAtsCandidatesRequestBodyAdditionalPhoneNumber]
    ] = None

    location: Optional[PostAtsCandidatesRequestBodyLocation] = None
    r"""The location of the candidate."""

    gender: Optional[PostAtsCandidatesRequestBodyGender] = None
    r"""The gender of the candidate. Must be one of `MALE`, `FEMALE`, or `OTHER`."""

    availability_date: Optional[datetime] = None
    r"""The date the candidate is available to start working.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    salary_expectations: Optional[PostAtsCandidatesRequestBodySalaryExpectations] = None
    r"""The salary expectations of the applicant. We will automatically convert the amount to a format that is suitable for the ATS you are using. For example, if you are using monthly salary expectations, we will convert the amount to a yearly salary if the ATS expects yearly salary expectations."""

    social_links: Optional[List[PostAtsCandidatesRequestBodySocialLink]] = None
    r"""A list of social media links of the candidate. The links must be valid URLs."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "additional_email_addresses",
                "company",
                "title",
                "phone_number",
                "additional_phone_numbers",
                "location",
                "gender",
                "availability_date",
                "salary_expectations",
                "social_links",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyApplicationTypedDict(TypedDict):
    r"""Currently, every candidate has one application. If you are interested in talent pools, please contact Kombo."""

    job_id: str
    r"""The Kombo ID or Remote ID of the Job this candidate should apply to. If you want to use the ID of the integrated system (remote_id) you need to prefix the ID with \"remote:\". You can use the remote ID if you do not want to sync jobs."""
    stage_id: NotRequired[str]
    r"""Stage this candidate should be in. If left out, the default stage for this job will be used."""


class PostAtsCandidatesRequestBodyApplication(BaseModel):
    r"""Currently, every candidate has one application. If you are interested in talent pools, please contact Kombo."""

    job_id: str
    r"""The Kombo ID or Remote ID of the Job this candidate should apply to. If you want to use the ID of the integrated system (remote_id) you need to prefix the ID with \"remote:\". You can use the remote ID if you do not want to sync jobs."""

    stage_id: Optional[str] = None
    r"""Stage this candidate should be in. If left out, the default stage for this job will be used."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["stage_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyAnswerTypedDict(TypedDict):
    r"""Answer to a `FILE` question."""

    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""
    content_type: NotRequired[str]
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """
    data_url: NotRequired[str]
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """
    data: NotRequired[str]
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """


class PostAtsCandidatesRequestBodyAnswer(BaseModel):
    r"""Answer to a `FILE` question."""

    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""

    content_type: Optional[str] = None
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """

    data_url: Optional[str] = None
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    data: Optional[str] = None
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["content_type", "data_url", "data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


PostAtsCandidatesRequestBodyAnswerUnionTypedDict = TypeAliasType(
    "PostAtsCandidatesRequestBodyAnswerUnionTypedDict",
    Union[
        PostAtsCandidatesRequestBodyAnswerTypedDict,
        str,
        bool,
        float,
        List[str],
        datetime,
    ],
)
r"""Answer to a question. This will be validated based on the question format and throw an error if the answer is invalid. Here is a description of each question type and the required answer format:

`TEXT` - Simply provide a \"string\" answer.

`SINGLE_SELECT` - Provide the ID of the answer as a string.

`MULTI_SELECT` - Provide a string array containing the question IDs of the selected options.

`BOOLEAN` - Either `true` or `false`.

`NUMBER` - A number.

`DATE` - Provide the answer as an ISO 8601 date string.

`FILE` - Please select Option 6 in the dropdown above to see the required format.
"""


PostAtsCandidatesRequestBodyAnswerUnion = TypeAliasType(
    "PostAtsCandidatesRequestBodyAnswerUnion",
    Union[PostAtsCandidatesRequestBodyAnswer, str, bool, float, List[str], datetime],
)
r"""Answer to a question. This will be validated based on the question format and throw an error if the answer is invalid. Here is a description of each question type and the required answer format:

`TEXT` - Simply provide a \"string\" answer.

`SINGLE_SELECT` - Provide the ID of the answer as a string.

`MULTI_SELECT` - Provide a string array containing the question IDs of the selected options.

`BOOLEAN` - Either `true` or `false`.

`NUMBER` - A number.

`DATE` - Provide the answer as an ISO 8601 date string.

`FILE` - Please select Option 6 in the dropdown above to see the required format.
"""


class PostAtsCandidatesRequestBodyScreeningQuestionAnswerTypedDict(TypedDict):
    question_id: str
    r"""ID of the question returned by the Kombo API. We'll report a warning in the logs if the question can't be found on the job."""
    answer: PostAtsCandidatesRequestBodyAnswerUnionTypedDict
    r"""Answer to a question. This will be validated based on the question format and throw an error if the answer is invalid. Here is a description of each question type and the required answer format:

    `TEXT` - Simply provide a \"string\" answer.

    `SINGLE_SELECT` - Provide the ID of the answer as a string.

    `MULTI_SELECT` - Provide a string array containing the question IDs of the selected options.

    `BOOLEAN` - Either `true` or `false`.

    `NUMBER` - A number.

    `DATE` - Provide the answer as an ISO 8601 date string.

    `FILE` - Please select Option 6 in the dropdown above to see the required format.
    """


class PostAtsCandidatesRequestBodyScreeningQuestionAnswer(BaseModel):
    question_id: str
    r"""ID of the question returned by the Kombo API. We'll report a warning in the logs if the question can't be found on the job."""

    answer: PostAtsCandidatesRequestBodyAnswerUnion
    r"""Answer to a question. This will be validated based on the question format and throw an error if the answer is invalid. Here is a description of each question type and the required answer format:

    `TEXT` - Simply provide a \"string\" answer.

    `SINGLE_SELECT` - Provide the ID of the answer as a string.

    `MULTI_SELECT` - Provide a string array containing the question IDs of the selected options.

    `BOOLEAN` - Either `true` or `false`.

    `NUMBER` - A number.

    `DATE` - Provide the answer as an ISO 8601 date string.

    `FILE` - Please select Option 6 in the dropdown above to see the required format.
    """


PostAtsCandidatesRequestBodyAttachmentType = Literal[
    "CV",
    "COVER_LETTER",
    "OTHER",
]
r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""


class PostAtsCandidatesRequestBodyAttachmentTypedDict(TypedDict):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""
    type: PostAtsCandidatesRequestBodyAttachmentType
    r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""
    content_type: NotRequired[str]
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """
    data_url: NotRequired[str]
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """
    data: NotRequired[str]
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """


class PostAtsCandidatesRequestBodyAttachment(BaseModel):
    name: str
    r"""File name of the file you want to upload. We recommend providing something human-readable with a valid file extension (like `Resume.pdf`), as this might be shown in your customer's system."""

    type: PostAtsCandidatesRequestBodyAttachmentType
    r"""Type of the attachment in the ATS. This may affect where the uploaded file will be shown in your customer's system."""

    content_type: Optional[str] = None
    r"""Content/MIME type of the file (e.g., `application/pdf`).

    If you provide `data`, this is required. If you provide `data_url`, this is optional and we'll attempt to use the `Content-Type` header of the response.

    **Note:** Please validate that the content type you provide is actually meaningful (and not something generic like [`application/octet-stream`](https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/MIME_types#applicationoctet-stream)). Especially when using object storage (like AWS S3), it's easy to accidentally discard the content types of user-provided files, so make sure to explicitly persist them when processing uploads from your users (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingMetadata.html#SysMetadata), [Google Cloud](https://cloud.google.com/storage/docs/metadata#content-type), and [Azure](https://learn.microsoft.com/en-us/rest/api/storageservices/put-blob#request-headers-all-blob-types)).
    """

    data_url: Optional[str] = None
    r"""URL to the file you want to upload that can be accessed without authentication headers. We will download the content immediately when receiving the request, so the URL can be short-lived.

    If you're using an object storage provider (like AWS S3), we strongly recommend providing a signed URL for secure access (see guides for [AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ShareObjectPreSignedURL.html), [Google Cloud](https://cloud.google.com/storage/docs/access-control/signed-urls), and [Azure](https://learn.microsoft.com/en-us/azure/storage/common/storage-sas-overview)).

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    data: Optional[str] = None
    r"""Base64-encoded contents of the file you want to upload.

    **Note:** You must provide either this or `data_url`. We recommend `data_url` over `data` for most cases.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["content_type", "data_url", "data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class PostAtsCandidatesRequestBodySourceTypedDict(TypedDict):
    r"""**(⚠️ Deprecated - Use [automatic source writing](/ats/features/application-attribution#automatic-attribution) instead)** Optional source information that will be attached to the candidate. If
    you're a job board or recruiting service, you can use this to make sure your
    customers can see which candidates came from you.

    This is deprecated because writing sources requires users to do some setup in most ATSs.
    """

    name: NotRequired[str]
    r"""Name of the source (e.g., `\"Example Job Board\"`)."""
    unified_key: NotRequired[str]
    id: NotRequired[str]


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class PostAtsCandidatesRequestBodySource(BaseModel):
    r"""**(⚠️ Deprecated - Use [automatic source writing](/ats/features/application-attribution#automatic-attribution) instead)** Optional source information that will be attached to the candidate. If
    you're a job board or recruiting service, you can use this to make sure your
    customers can see which candidates came from you.

    This is deprecated because writing sources requires users to do some setup in most ATSs.
    """

    name: Optional[str] = None
    r"""Name of the source (e.g., `\"Example Job Board\"`)."""

    unified_key: Optional[str] = None

    id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["name", "unified_key", "id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodySourcedByTypedDict(TypedDict):
    r"""Credit the recruiter or team member who sourced this candidate.

    While the `source` field tracks the channel/platform (e.g., \"Awesome Jobboard\"), the `sourced_by` field tracks the individual person responsible for finding the candidate.
    """

    user_id: str
    r"""The Kombo ID or Remote ID of the User. Use the Kombo ID directly, or prefix the remote ID with \"remote:\" to reference the user by their ID in the remote system."""


class PostAtsCandidatesRequestBodySourcedBy(BaseModel):
    r"""Credit the recruiter or team member who sourced this candidate.

    While the `source` field tracks the channel/platform (e.g., \"Awesome Jobboard\"), the `sourced_by` field tracks the individual person responsible for finding the candidate.
    """

    user_id: str
    r"""The Kombo ID or Remote ID of the User. Use the Kombo ID directly, or prefix the remote ID with \"remote:\" to reference the user by their ID in the remote system."""


class PostAtsCandidatesRequestBodyGdprConsentTypedDict(TypedDict):
    r"""Optional GDPR consent information required in some jurisdictions (like the Czech Republic or Slovakia)."""

    expires_at: NotRequired[datetime]
    r"""Until when the candidate has granted the company they're applying to permission to process their personal data.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    given: NotRequired[bool]
    r"""Whether the candidate has given consent."""


class PostAtsCandidatesRequestBodyGdprConsent(BaseModel):
    r"""Optional GDPR consent information required in some jurisdictions (like the Czech Republic or Slovakia)."""

    expires_at: Optional[datetime] = None
    r"""Until when the candidate has granted the company they're applying to permission to process their personal data.
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    given: Optional[bool] = None
    r"""Whether the candidate has given consent."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["expires_at", "given"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodySuccessfactorsTypedDict(TypedDict):
    r"""Fields specific to SAP SuccessFactors."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to SuccessFactor's `Candidate` object."""
    job_application: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to SuccessFactor's `JobApplication` object."""
    copy_job_application_attachments: NotRequired[bool]
    r"""If set to true, we will copy custom attachments from the JobApplication to the Candidate."""
    update_existing_candidate: NotRequired[Nullable[bool]]
    r"""When the candidate already exists, whether to update the Candidate with the remote fields found under the Candidate entity."""


class PostAtsCandidatesRequestBodySuccessfactors(BaseModel):
    r"""Fields specific to SAP SuccessFactors."""

    candidate: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="Candidate")
    ] = None
    r"""Fields that we will pass through to SuccessFactor's `Candidate` object."""

    job_application: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="JobApplication")
    ] = None
    r"""Fields that we will pass through to SuccessFactor's `JobApplication` object."""

    copy_job_application_attachments: Annotated[
        Optional[bool], pydantic.Field(alias="copyJobApplicationAttachments")
    ] = None
    r"""If set to true, we will copy custom attachments from the JobApplication to the Candidate."""

    update_existing_candidate: OptionalNullable[bool] = UNSET
    r"""When the candidate already exists, whether to update the Candidate with the remote fields found under the Candidate entity."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "Candidate",
                "JobApplication",
                "copyJobApplicationAttachments",
                "update_existing_candidate",
            ]
        )
        nullable_fields = set(["update_existing_candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyPersonioTypedDict(TypedDict):
    r"""Fields specific to Personio."""

    application: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Personio's `application` object."""


class PostAtsCandidatesRequestBodyPersonio(BaseModel):
    r"""Fields specific to Personio."""

    application: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Personio's `application` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["application"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyTalentsoftTypedDict(TypedDict):
    r"""Fields specific to TalentSoft."""

    applicant: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to TalentSoft's `applicant` object."""
    application: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to TalentSoft's `application` object."""


class PostAtsCandidatesRequestBodyTalentsoft(BaseModel):
    r"""Fields specific to TalentSoft."""

    applicant: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to TalentSoft's `applicant` object."""

    application: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to TalentSoft's `application` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["applicant", "application"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyTeamtailorApplicationTypedDict(TypedDict):
    attributes: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Teamtailor's attributes section `Job application` object."""


class PostAtsCandidatesRequestBodyTeamtailorApplication(BaseModel):
    attributes: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Teamtailor's attributes section `Job application` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["attributes"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyTeamtailorTypedDict(TypedDict):
    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Teamtailor's `Candidate` object."""
    application: NotRequired[PostAtsCandidatesRequestBodyTeamtailorApplicationTypedDict]


class PostAtsCandidatesRequestBodyTeamtailor(BaseModel):
    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Teamtailor's `Candidate` object."""

    application: Optional[PostAtsCandidatesRequestBodyTeamtailorApplication] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate", "application"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyPostHeadersTypedDict(TypedDict):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: NotRequired[Nullable[str]]
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""


class PostAtsCandidatesRequestBodyPostHeaders(BaseModel):
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    on_behalf_of: Annotated[
        OptionalNullable[str], pydantic.Field(alias="On-Behalf-Of")
    ] = UNSET
    r"""ID of the the user that will show up as having performed the action in Greenhouse. We already pass a value by default, but you can use this to override it."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["On-Behalf-Of"])
        nullable_fields = set(["On-Behalf-Of"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyGreenhouseTypedDict(TypedDict):
    r"""Fields specific to Greenhouse."""

    post_headers: NotRequired[PostAtsCandidatesRequestBodyPostHeadersTypedDict]
    r"""Headers we will pass with `POST` requests to Greenhouse."""


class PostAtsCandidatesRequestBodyGreenhouse(BaseModel):
    r"""Fields specific to Greenhouse."""

    post_headers: Optional[PostAtsCandidatesRequestBodyPostHeaders] = None
    r"""Headers we will pass with `POST` requests to Greenhouse."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["post_headers"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLeverTypedDict(TypedDict):
    r"""Fields specific to Lever."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Lever's `Candidate` object. Note: make sure to submit the keys and values in the correct form data format."""


class PostAtsCandidatesRequestBodyLever(BaseModel):
    r"""Fields specific to Lever."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Lever's `Candidate` object. Note: make sure to submit the keys and values in the correct form data format."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyWorkableTypedDict(TypedDict):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: NotRequired[str]
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""


class PostAtsCandidatesRequestBodyWorkable(BaseModel):
    r"""Workable specific remote fields for ATS actions."""

    on_behalf_of_user_remote_id: Optional[str] = None
    r"""The remote ID of the user that will be displayed in the UI as the one that performed the action."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["on_behalf_of_user_remote_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodySocialSuffixReferenceTypedDict(TypedDict):
    r"""The WID of the social suffix reference for the candidate"""

    predefined_name_component_id: str


class PostAtsCandidatesRequestBodySocialSuffixReference(BaseModel):
    r"""The WID of the social suffix reference for the candidate"""

    predefined_name_component_id: Annotated[
        str, pydantic.Field(alias="Predefined_Name_Component_ID")
    ]


class PostAtsCandidatesRequestBodyNameDetailDataTypedDict(TypedDict):
    middle_name: NotRequired[str]
    r"""The candidate's middle name"""
    social_suffix_reference: NotRequired[
        PostAtsCandidatesRequestBodySocialSuffixReferenceTypedDict
    ]
    r"""The WID of the social suffix reference for the candidate"""


class PostAtsCandidatesRequestBodyNameDetailData(BaseModel):
    middle_name: Annotated[Optional[str], pydantic.Field(alias="Middle_Name")] = None
    r"""The candidate's middle name"""

    social_suffix_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodySocialSuffixReference],
        pydantic.Field(alias="Social_Suffix_Reference"),
    ] = None
    r"""The WID of the social suffix reference for the candidate"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Middle_Name", "Social_Suffix_Reference"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageReferenceTypedDict(TypedDict):
    r"""Used to set the candidate's primary language"""

    wid: str


class PostAtsCandidatesRequestBodyLanguageReference(BaseModel):
    r"""Used to set the candidate's primary language"""

    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyGlobalPersonalInformationDataTypedDict(TypedDict):
    date_of_birth: NotRequired[str]


class PostAtsCandidatesRequestBodyGlobalPersonalInformationData(BaseModel):
    date_of_birth: Annotated[Optional[str], pydantic.Field(alias="Date_of_Birth")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Date_of_Birth"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyJobAppliedToDataTypedDict(TypedDict):
    global_personal_information_data: NotRequired[
        PostAtsCandidatesRequestBodyGlobalPersonalInformationDataTypedDict
    ]


class PostAtsCandidatesRequestBodyJobAppliedToData(BaseModel):
    global_personal_information_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyGlobalPersonalInformationData],
        pydantic.Field(alias="Global_Personal_Information_Data"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Global_Personal_Information_Data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyFieldOfStudyReferenceTypedDict(TypedDict):
    wid: str


class PostAtsCandidatesRequestBodyFieldOfStudyReference(BaseModel):
    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyDegreeReferenceTypedDict(TypedDict):
    wid: str


class PostAtsCandidatesRequestBodyDegreeReference(BaseModel):
    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyEducationDatumTypedDict(TypedDict):
    school_name: NotRequired[str]
    first_year_attended: NotRequired[float]
    last_year_attended: NotRequired[float]
    field_of_study_reference: NotRequired[
        PostAtsCandidatesRequestBodyFieldOfStudyReferenceTypedDict
    ]
    degree_reference: NotRequired[PostAtsCandidatesRequestBodyDegreeReferenceTypedDict]
    grade_average: NotRequired[str]


class PostAtsCandidatesRequestBodyEducationDatum(BaseModel):
    school_name: Annotated[Optional[str], pydantic.Field(alias="School_Name")] = None

    first_year_attended: Annotated[
        Optional[float], pydantic.Field(alias="First_Year_Attended")
    ] = None

    last_year_attended: Annotated[
        Optional[float], pydantic.Field(alias="Last_Year_Attended")
    ] = None

    field_of_study_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyFieldOfStudyReference],
        pydantic.Field(alias="Field_of_Study_Reference"),
    ] = None

    degree_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyDegreeReference],
        pydantic.Field(alias="Degree_Reference"),
    ] = None

    grade_average: Annotated[Optional[str], pydantic.Field(alias="Grade_Average")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "School_Name",
                "First_Year_Attended",
                "Last_Year_Attended",
                "Field_of_Study_Reference",
                "Degree_Reference",
                "Grade_Average",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodySkillDatumTypedDict(TypedDict):
    skill_name: NotRequired[str]


class PostAtsCandidatesRequestBodySkillDatum(BaseModel):
    skill_name: Annotated[Optional[str], pydantic.Field(alias="Skill_Name")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Skill_Name"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageDatumLanguageReferenceTypedDict(TypedDict):
    wid: NotRequired[str]


class PostAtsCandidatesRequestBodyLanguageDatumLanguageReference(BaseModel):
    wid: Annotated[Optional[str], pydantic.Field(alias="WID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["WID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageProficiencyReferenceTypedDict(TypedDict):
    wid: str


class PostAtsCandidatesRequestBodyLanguageProficiencyReference(BaseModel):
    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyLanguageAbilityTypeReferenceTypedDict(TypedDict):
    wid: str


class PostAtsCandidatesRequestBodyLanguageAbilityTypeReference(BaseModel):
    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyLanguageAbilityDataTypedDict(TypedDict):
    language_proficiency_reference: NotRequired[
        PostAtsCandidatesRequestBodyLanguageProficiencyReferenceTypedDict
    ]
    language_ability_type_reference: NotRequired[
        PostAtsCandidatesRequestBodyLanguageAbilityTypeReferenceTypedDict
    ]


class PostAtsCandidatesRequestBodyLanguageAbilityData(BaseModel):
    language_proficiency_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguageProficiencyReference],
        pydantic.Field(alias="Language_Proficiency_Reference"),
    ] = None

    language_ability_type_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguageAbilityTypeReference],
        pydantic.Field(alias="Language_Ability_Type_Reference"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["Language_Proficiency_Reference", "Language_Ability_Type_Reference"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageAbilityTypedDict(TypedDict):
    language_ability_data: NotRequired[
        PostAtsCandidatesRequestBodyLanguageAbilityDataTypedDict
    ]


class PostAtsCandidatesRequestBodyLanguageAbility(BaseModel):
    language_ability_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguageAbilityData],
        pydantic.Field(alias="Language_Ability_Data"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Language_Ability_Data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageTypedDict(TypedDict):
    language_ability: List[PostAtsCandidatesRequestBodyLanguageAbilityTypedDict]
    native: NotRequired[bool]


class PostAtsCandidatesRequestBodyLanguage(BaseModel):
    language_ability: Annotated[
        List[PostAtsCandidatesRequestBodyLanguageAbility],
        pydantic.Field(alias="Language_Ability"),
    ]

    native: Annotated[Optional[bool], pydantic.Field(alias="Native")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Native"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyLanguageDatumTypedDict(TypedDict):
    language_reference: NotRequired[
        PostAtsCandidatesRequestBodyLanguageDatumLanguageReferenceTypedDict
    ]
    language: NotRequired[PostAtsCandidatesRequestBodyLanguageTypedDict]


class PostAtsCandidatesRequestBodyLanguageDatum(BaseModel):
    language_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguageDatumLanguageReference],
        pydantic.Field(alias="Language_Reference"),
    ] = None

    language: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguage], pydantic.Field(alias="Language")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Language_Reference", "Language"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyExperienceDatumTypedDict(TypedDict):
    company_name: str
    title: str
    start_date: datetime
    r"""YYYY-MM-DDTHH:mm:ss.sssZ
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    location: NotRequired[str]
    end_date: NotRequired[datetime]
    r"""YYYY-MM-DDTHH:mm:ss.sssZ
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """
    currently_work_here: NotRequired[bool]
    description: NotRequired[str]


class PostAtsCandidatesRequestBodyExperienceDatum(BaseModel):
    company_name: Annotated[str, pydantic.Field(alias="Company_Name")]

    title: Annotated[str, pydantic.Field(alias="Title")]

    start_date: Annotated[datetime, pydantic.Field(alias="Start_Date")]
    r"""YYYY-MM-DDTHH:mm:ss.sssZ
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    location: Annotated[Optional[str], pydantic.Field(alias="Location")] = None

    end_date: Annotated[Optional[datetime], pydantic.Field(alias="End_Date")] = None
    r"""YYYY-MM-DDTHH:mm:ss.sssZ
    https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString
    """

    currently_work_here: Annotated[
        Optional[bool], pydantic.Field(alias="Currently_Work_Here")
    ] = None

    description: Annotated[Optional[str], pydantic.Field(alias="Description")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["Location", "End_Date", "Currently_Work_Here", "Description"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyResumeDataTypedDict(TypedDict):
    education_data: NotRequired[
        List[PostAtsCandidatesRequestBodyEducationDatumTypedDict]
    ]
    skill_data: NotRequired[List[PostAtsCandidatesRequestBodySkillDatumTypedDict]]
    language_data: NotRequired[List[PostAtsCandidatesRequestBodyLanguageDatumTypedDict]]
    experience_data: NotRequired[
        List[PostAtsCandidatesRequestBodyExperienceDatumTypedDict]
    ]


class PostAtsCandidatesRequestBodyResumeData(BaseModel):
    education_data: Annotated[
        Optional[List[PostAtsCandidatesRequestBodyEducationDatum]],
        pydantic.Field(alias="Education_Data"),
    ] = None

    skill_data: Annotated[
        Optional[List[PostAtsCandidatesRequestBodySkillDatum]],
        pydantic.Field(alias="Skill_Data"),
    ] = None

    language_data: Annotated[
        Optional[List[PostAtsCandidatesRequestBodyLanguageDatum]],
        pydantic.Field(alias="Language_Data"),
    ] = None

    experience_data: Annotated[
        Optional[List[PostAtsCandidatesRequestBodyExperienceDatum]],
        pydantic.Field(alias="Experience_Data"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["Education_Data", "Skill_Data", "Language_Data", "Experience_Data"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyJobApplicationDataTypedDict(TypedDict):
    job_applied_to_data: NotRequired[
        PostAtsCandidatesRequestBodyJobAppliedToDataTypedDict
    ]
    resume_data: NotRequired[PostAtsCandidatesRequestBodyResumeDataTypedDict]


class PostAtsCandidatesRequestBodyJobApplicationData(BaseModel):
    job_applied_to_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyJobAppliedToData],
        pydantic.Field(alias="Job_Applied_To_Data"),
    ] = None

    resume_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyResumeData],
        pydantic.Field(alias="Resume_Data"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Job_Applied_To_Data", "Resume_Data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyCountryRegionReferenceTypedDict(TypedDict):
    country_region_id: str


class PostAtsCandidatesRequestBodyCountryRegionReference(BaseModel):
    country_region_id: Annotated[str, pydantic.Field(alias="Country_Region_ID")]


class PostAtsCandidatesRequestBodyCountryCityReferenceTypedDict(TypedDict):
    wid: str


class PostAtsCandidatesRequestBodyCountryCityReference(BaseModel):
    wid: Annotated[str, pydantic.Field(alias="WID")]


class PostAtsCandidatesRequestBodyLocationDataTypedDict(TypedDict):
    address_line_1: NotRequired[str]
    address_line_2: NotRequired[str]
    region_subdivision_1: NotRequired[str]
    country_region_reference: NotRequired[
        PostAtsCandidatesRequestBodyCountryRegionReferenceTypedDict
    ]
    country_city_reference: NotRequired[
        PostAtsCandidatesRequestBodyCountryCityReferenceTypedDict
    ]


class PostAtsCandidatesRequestBodyLocationData(BaseModel):
    address_line_1: Annotated[Optional[str], pydantic.Field(alias="Address_Line_1")] = (
        None
    )

    address_line_2: Annotated[Optional[str], pydantic.Field(alias="Address_Line_2")] = (
        None
    )

    region_subdivision_1: Annotated[
        Optional[str], pydantic.Field(alias="Region_Subdivision_1")
    ] = None

    country_region_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyCountryRegionReference],
        pydantic.Field(alias="Country_Region_Reference"),
    ] = None

    country_city_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyCountryCityReference],
        pydantic.Field(alias="Country_City_Reference"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "Address_Line_1",
                "Address_Line_2",
                "Region_Subdivision_1",
                "Country_Region_Reference",
                "Country_City_Reference",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyContactDataTypedDict(TypedDict):
    location_data: NotRequired[PostAtsCandidatesRequestBodyLocationDataTypedDict]


class PostAtsCandidatesRequestBodyContactData(BaseModel):
    location_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyLocationData],
        pydantic.Field(alias="Location_Data"),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Location_Data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyWorkerReferenceTypedDict(TypedDict):
    r"""Reference to the Worker (employee) to link the candidate to. Provide either WID or Employee_ID."""

    wid: NotRequired[str]
    employee_id: NotRequired[str]


class PostAtsCandidatesRequestBodyWorkerReference(BaseModel):
    r"""Reference to the Worker (employee) to link the candidate to. Provide either WID or Employee_ID."""

    wid: Annotated[Optional[str], pydantic.Field(alias="WID")] = None

    employee_id: Annotated[Optional[str], pydantic.Field(alias="Employee_ID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["WID", "Employee_ID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyCandidateDataTypedDict(TypedDict):
    name_detail_data: NotRequired[PostAtsCandidatesRequestBodyNameDetailDataTypedDict]
    language_reference: NotRequired[
        PostAtsCandidatesRequestBodyLanguageReferenceTypedDict
    ]
    r"""Used to set the candidate's primary language"""
    job_application_data: NotRequired[
        PostAtsCandidatesRequestBodyJobApplicationDataTypedDict
    ]
    contact_data: NotRequired[PostAtsCandidatesRequestBodyContactDataTypedDict]
    worker_reference: NotRequired[PostAtsCandidatesRequestBodyWorkerReferenceTypedDict]
    r"""Reference to the Worker (employee) to link the candidate to. Provide either WID or Employee_ID."""


class PostAtsCandidatesRequestBodyCandidateData(BaseModel):
    name_detail_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyNameDetailData],
        pydantic.Field(alias="Name_Detail_Data"),
    ] = None

    language_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyLanguageReference],
        pydantic.Field(alias="Language_Reference"),
    ] = None
    r"""Used to set the candidate's primary language"""

    job_application_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyJobApplicationData],
        pydantic.Field(alias="Job_Application_Data"),
    ] = None

    contact_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyContactData],
        pydantic.Field(alias="Contact_Data"),
    ] = None

    worker_reference: Annotated[
        Optional[PostAtsCandidatesRequestBodyWorkerReference],
        pydantic.Field(alias="Worker_Reference"),
    ] = None
    r"""Reference to the Worker (employee) to link the candidate to. Provide either WID or Employee_ID."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "Name_Detail_Data",
                "Language_Reference",
                "Job_Application_Data",
                "Contact_Data",
                "Worker_Reference",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyWorkdayTypedDict(TypedDict):
    r"""Fields specific to Workday. The remote fields schema follows the documentation at https://community.workday.com/sites/default/files/file-hosting/productionapi/Recruiting/v43.0/Put_Candidate.html. Only defined fields are supported, if you need additional field support please reach out to Kombo support."""

    candidate_data: NotRequired[PostAtsCandidatesRequestBodyCandidateDataTypedDict]
    override_source_reference_wid: NotRequired[str]
    r"""Used to override the automatic source WID."""


class PostAtsCandidatesRequestBodyWorkday(BaseModel):
    r"""Fields specific to Workday. The remote fields schema follows the documentation at https://community.workday.com/sites/default/files/file-hosting/productionapi/Recruiting/v43.0/Put_Candidate.html. Only defined fields are supported, if you need additional field support please reach out to Kombo support."""

    candidate_data: Annotated[
        Optional[PostAtsCandidatesRequestBodyCandidateData],
        pydantic.Field(alias="Candidate_Data"),
    ] = None

    override_source_reference_wid: Annotated[
        Optional[str], pydantic.Field(alias="Override_Source_Reference_WID")
    ] = None
    r"""Used to override the automatic source WID."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["Candidate_Data", "Override_Source_Reference_WID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyZohorecruitTypedDict(TypedDict):
    r"""Fields specific to Zoho Recruit."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Zoho Recruit's `Candidate` object."""


class PostAtsCandidatesRequestBodyZohorecruit(BaseModel):
    r"""Fields specific to Zoho Recruit."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Zoho Recruit's `Candidate` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyBullhornTypedDict(TypedDict):
    r"""Fields specific to Bullhorn."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Bullhorn's `Candidate` object."""
    job_submission: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Bullhorn's `JobSubmission` object."""


class PostAtsCandidatesRequestBodyBullhorn(BaseModel):
    r"""Fields specific to Bullhorn."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Bullhorn's `Candidate` object."""

    job_submission: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Bullhorn's `JobSubmission` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate", "job_submission"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyConsentDecisionsTypedDict(TypedDict):
    r"""The consent decisions for the candidate. SmartRecruiters supports two consent models: 'Single' (use the `SINGLE` key) and 'Separated' (use `SMART_RECRUIT`, `SMART_CRM`, `SMART_MESSAGE_SMS`, and/or `SMART_MESSAGE_WHATSAPP` keys). When this field is provided, it takes precedence over the `gdpr_consent` field for the `consentDecisions` property. See: https://developers.smartrecruiters.com/docs/partners-post-an-application"""

    single: NotRequired[bool]
    smart_recruit: NotRequired[bool]
    smart_crm: NotRequired[bool]
    smart_message_sms: NotRequired[bool]
    smart_message_whatsapp: NotRequired[bool]


class PostAtsCandidatesRequestBodyConsentDecisions(BaseModel):
    r"""The consent decisions for the candidate. SmartRecruiters supports two consent models: 'Single' (use the `SINGLE` key) and 'Separated' (use `SMART_RECRUIT`, `SMART_CRM`, `SMART_MESSAGE_SMS`, and/or `SMART_MESSAGE_WHATSAPP` keys). When this field is provided, it takes precedence over the `gdpr_consent` field for the `consentDecisions` property. See: https://developers.smartrecruiters.com/docs/partners-post-an-application"""

    single: Annotated[Optional[bool], pydantic.Field(alias="SINGLE")] = None

    smart_recruit: Annotated[Optional[bool], pydantic.Field(alias="SMART_RECRUIT")] = (
        None
    )

    smart_crm: Annotated[Optional[bool], pydantic.Field(alias="SMART_CRM")] = None

    smart_message_sms: Annotated[
        Optional[bool], pydantic.Field(alias="SMART_MESSAGE_SMS")
    ] = None

    smart_message_whatsapp: Annotated[
        Optional[bool], pydantic.Field(alias="SMART_MESSAGE_WHATSAPP")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "SINGLE",
                "SMART_RECRUIT",
                "SMART_CRM",
                "SMART_MESSAGE_SMS",
                "SMART_MESSAGE_WHATSAPP",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodySmartrecruitersTypedDict(TypedDict):
    r"""Fields specific to SmartRecruiters."""

    candidate_with_questions: NotRequired[Dict[str, Any]]
    r"""**(⚠️ Deprecated - Use the `candidate` field instead.)** Fields that we will pass through to the SmartRecruiters's `Candidate` object when created with screening question answers. This API is used: https://developers.smartrecruiters.com/reference/createcandidate-1"""
    candidate_without_questions: NotRequired[Dict[str, Any]]
    r"""**(⚠️ Deprecated - Use the `candidate` field instead.)** Fields that we will pass through to the SmartRecruiters's `Candidate` object when created with screening question answers. This API is used: https://developers.smartrecruiters.com/reference/candidatesaddtojob-1"""
    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to the SmartRecruiters's `Candidate` object. This API is used: https://developers.smartrecruiters.com/reference/createcandidate-1"""
    consent_decisions: NotRequired[
        PostAtsCandidatesRequestBodyConsentDecisionsTypedDict
    ]
    r"""The consent decisions for the candidate. SmartRecruiters supports two consent models: 'Single' (use the `SINGLE` key) and 'Separated' (use `SMART_RECRUIT`, `SMART_CRM`, `SMART_MESSAGE_SMS`, and/or `SMART_MESSAGE_WHATSAPP` keys). When this field is provided, it takes precedence over the `gdpr_consent` field for the `consentDecisions` property. See: https://developers.smartrecruiters.com/docs/partners-post-an-application"""


class PostAtsCandidatesRequestBodySmartrecruiters(BaseModel):
    r"""Fields specific to SmartRecruiters."""

    candidate_with_questions: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""**(⚠️ Deprecated - Use the `candidate` field instead.)** Fields that we will pass through to the SmartRecruiters's `Candidate` object when created with screening question answers. This API is used: https://developers.smartrecruiters.com/reference/createcandidate-1"""

    candidate_without_questions: Annotated[
        Optional[Dict[str, Any]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""**(⚠️ Deprecated - Use the `candidate` field instead.)** Fields that we will pass through to the SmartRecruiters's `Candidate` object when created with screening question answers. This API is used: https://developers.smartrecruiters.com/reference/candidatesaddtojob-1"""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to the SmartRecruiters's `Candidate` object. This API is used: https://developers.smartrecruiters.com/reference/createcandidate-1"""

    consent_decisions: Optional[PostAtsCandidatesRequestBodyConsentDecisions] = None
    r"""The consent decisions for the candidate. SmartRecruiters supports two consent models: 'Single' (use the `SINGLE` key) and 'Separated' (use `SMART_RECRUIT`, `SMART_CRM`, `SMART_MESSAGE_SMS`, and/or `SMART_MESSAGE_WHATSAPP` keys). When this field is provided, it takes precedence over the `gdpr_consent` field for the `consentDecisions` property. See: https://developers.smartrecruiters.com/docs/partners-post-an-application"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "candidate_with_questions",
                "candidate_without_questions",
                "candidate",
                "consent_decisions",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyTalentadoreTypedDict(TypedDict):
    r"""Fields specific to Talentadore."""

    applications: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to the Talentadore's when creating applications."""


class PostAtsCandidatesRequestBodyTalentadore(BaseModel):
    r"""Fields specific to Talentadore."""

    applications: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to the Talentadore's when creating applications."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["applications"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyGuidecomTypedDict(TypedDict):
    r"""Fields specific to GuideCom."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to GuideCom's `Candidate` object."""


class PostAtsCandidatesRequestBodyGuidecom(BaseModel):
    r"""Fields specific to GuideCom."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to GuideCom's `Candidate` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyDvinciTypedDict(TypedDict):
    r"""Fields specific to d.vinci."""

    application: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to d.vinci's application object. This API is used: https://static.dvinci-easy.com/files/d.vinci%20application-apply-api.html#jobs__id__applyApi_post"""
    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to d.vinci's candidate/application payload as top-level fields (e.g., dateOfBirth)."""


class PostAtsCandidatesRequestBodyDvinci(BaseModel):
    r"""Fields specific to d.vinci."""

    application: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to d.vinci's application object. This API is used: https://static.dvinci-easy.com/files/d.vinci%20application-apply-api.html#jobs__id__applyApi_post"""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to d.vinci's candidate/application payload as top-level fields (e.g., dateOfBirth)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["application", "candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyHrworksTypedDict(TypedDict):
    r"""Fields specific to GuideCom."""

    job_application: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to HRWorks's `Job Application` object. This API is used: https://developers.hrworks.de/docs/hrworks-api-v2/53021f035f62d-post-job-applications"""


class PostAtsCandidatesRequestBodyHrworks(BaseModel):
    r"""Fields specific to GuideCom."""

    job_application: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="jobApplication")
    ] = None
    r"""Fields that we will pass through to HRWorks's `Job Application` object. This API is used: https://developers.hrworks.de/docs/hrworks-api-v2/53021f035f62d-post-job-applications"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["jobApplication"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyJobylonApplicationTypedDict(TypedDict):
    r"""Fields that we will pass through to Jobylon's create application [endpoint](https://developer.jobylon.com/push-api-and-webhooks#-xL0v)'s request body."""

    message: NotRequired[str]
    r"""The `message` field of Jobylon's create application endpoint's request body."""


class PostAtsCandidatesRequestBodyJobylonApplication(BaseModel):
    r"""Fields that we will pass through to Jobylon's create application [endpoint](https://developer.jobylon.com/push-api-and-webhooks#-xL0v)'s request body."""

    message: Optional[str] = None
    r"""The `message` field of Jobylon's create application endpoint's request body."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyJobylonTypedDict(TypedDict):
    r"""Fields specific to Jobylon."""

    application: NotRequired[PostAtsCandidatesRequestBodyJobylonApplicationTypedDict]
    r"""Fields that we will pass through to Jobylon's create application [endpoint](https://developer.jobylon.com/push-api-and-webhooks#-xL0v)'s request body."""


class PostAtsCandidatesRequestBodyJobylon(BaseModel):
    r"""Fields specific to Jobylon."""

    application: Optional[PostAtsCandidatesRequestBodyJobylonApplication] = None
    r"""Fields that we will pass through to Jobylon's create application [endpoint](https://developer.jobylon.com/push-api-and-webhooks#-xL0v)'s request body."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["application"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyStepTypedDict(TypedDict):
    id: int
    r"""The ID of the workflow step to assign to the candidate."""


class PostAtsCandidatesRequestBodyStep(BaseModel):
    id: int
    r"""The ID of the workflow step to assign to the candidate."""


class PostAtsCandidatesRequestBodyWorkflowTypedDict(TypedDict):
    step: NotRequired[PostAtsCandidatesRequestBodyStepTypedDict]


class PostAtsCandidatesRequestBodyWorkflow(BaseModel):
    step: Optional[PostAtsCandidatesRequestBodyStep] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["step"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyAvatureTypedDict(TypedDict):
    r"""Fields specific to Avature."""

    workflow: NotRequired[PostAtsCandidatesRequestBodyWorkflowTypedDict]


class PostAtsCandidatesRequestBodyAvature(BaseModel):
    r"""Fields specific to Avature."""

    workflow: Optional[PostAtsCandidatesRequestBodyWorkflow] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["workflow"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyRecruiteeCandidateTypedDict(TypedDict):
    cover_letter_text: NotRequired[str]
    r"""The cover letter text as a string. This will be visible on the main candidate page. Can be provided together with the `cover_letter` attachment, which will end up in a separate `file` section."""


class PostAtsCandidatesRequestBodyRecruiteeCandidate(BaseModel):
    cover_letter_text: Optional[str] = None
    r"""The cover letter text as a string. This will be visible on the main candidate page. Can be provided together with the `cover_letter` attachment, which will end up in a separate `file` section."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["cover_letter_text"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyRecruiteeTypedDict(TypedDict):
    r"""Fields specific to Recruitee."""

    candidate: NotRequired[PostAtsCandidatesRequestBodyRecruiteeCandidateTypedDict]


class PostAtsCandidatesRequestBodyRecruitee(BaseModel):
    r"""Fields specific to Recruitee."""

    candidate: Optional[PostAtsCandidatesRequestBodyRecruiteeCandidate] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyRexxTypedDict(TypedDict):
    r"""Fields specific to Rexx."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Rexx's application form fields."""


class PostAtsCandidatesRequestBodyRexx(BaseModel):
    r"""Fields specific to Rexx."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Rexx's application form fields."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyUmantisTypedDict(TypedDict):
    r"""Fields specific to Abacus Umantis."""

    person: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Abacus Umantis's \"Create a person\" endpoint's `attributes` when creating a candidate."""


class PostAtsCandidatesRequestBodyUmantis(BaseModel):
    r"""Fields specific to Abacus Umantis."""

    person: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Abacus Umantis's \"Create a person\" endpoint's `attributes` when creating a candidate."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["person"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyPilogaCandidateTypedDict(TypedDict):
    r"""Additional candidate fields for P&I Loga that will be mapped to the application form."""

    street: NotRequired[str]
    r"""The street address of the candidate."""


class PostAtsCandidatesRequestBodyPilogaCandidate(BaseModel):
    r"""Additional candidate fields for P&I Loga that will be mapped to the application form."""

    street: Optional[str] = None
    r"""The street address of the candidate."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["street"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyPilogaTypedDict(TypedDict):
    r"""Fields specific to P&I Loga."""

    candidate: NotRequired[PostAtsCandidatesRequestBodyPilogaCandidateTypedDict]
    r"""Additional candidate fields for P&I Loga that will be mapped to the application form."""


class PostAtsCandidatesRequestBodyPiloga(BaseModel):
    r"""Fields specific to P&I Loga."""

    candidate: Optional[PostAtsCandidatesRequestBodyPilogaCandidate] = None
    r"""Additional candidate fields for P&I Loga that will be mapped to the application form."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyPinpointTypedDict(TypedDict):
    r"""Fields specific to Pinpoint."""

    candidate: NotRequired[Dict[str, Any]]
    r"""Fields that we will pass through to Pinpoint's `Candidate` object."""


class PostAtsCandidatesRequestBodyPinpoint(BaseModel):
    r"""Fields specific to Pinpoint."""

    candidate: Optional[Dict[str, Any]] = None
    r"""Fields that we will pass through to Pinpoint's `Candidate` object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyCovetorestCandidateTypedDict(TypedDict):
    r"""Additional candidate fields that will be passed to the Coveto candidate creation."""

    mandant: NotRequired[float]
    r"""The mandant field for the candidate in Coveto."""


class PostAtsCandidatesRequestBodyCovetorestCandidate(BaseModel):
    r"""Additional candidate fields that will be passed to the Coveto candidate creation."""

    mandant: Optional[float] = None
    r"""The mandant field for the candidate in Coveto."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["mandant"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyCovetorestTypedDict(TypedDict):
    r"""Fields specific to Coveto REST."""

    candidate: NotRequired[PostAtsCandidatesRequestBodyCovetorestCandidateTypedDict]
    r"""Additional candidate fields that will be passed to the Coveto candidate creation."""


class PostAtsCandidatesRequestBodyCovetorest(BaseModel):
    r"""Fields specific to Coveto REST."""

    candidate: Optional[PostAtsCandidatesRequestBodyCovetorestCandidate] = None
    r"""Additional candidate fields that will be passed to the Coveto candidate creation."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["candidate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyRemoteFieldsTypedDict(TypedDict):
    r"""Additional fields that we will pass through to specific ATS systems."""

    successfactors: NotRequired[PostAtsCandidatesRequestBodySuccessfactorsTypedDict]
    r"""Fields specific to SAP SuccessFactors."""
    personio: NotRequired[PostAtsCandidatesRequestBodyPersonioTypedDict]
    r"""Fields specific to Personio."""
    talentsoft: NotRequired[PostAtsCandidatesRequestBodyTalentsoftTypedDict]
    r"""Fields specific to TalentSoft."""
    teamtailor: NotRequired[PostAtsCandidatesRequestBodyTeamtailorTypedDict]
    greenhouse: NotRequired[PostAtsCandidatesRequestBodyGreenhouseTypedDict]
    r"""Fields specific to Greenhouse."""
    lever: NotRequired[PostAtsCandidatesRequestBodyLeverTypedDict]
    r"""Fields specific to Lever."""
    workable: NotRequired[PostAtsCandidatesRequestBodyWorkableTypedDict]
    r"""Workable specific remote fields for ATS actions."""
    workday: NotRequired[PostAtsCandidatesRequestBodyWorkdayTypedDict]
    r"""Fields specific to Workday. The remote fields schema follows the documentation at https://community.workday.com/sites/default/files/file-hosting/productionapi/Recruiting/v43.0/Put_Candidate.html. Only defined fields are supported, if you need additional field support please reach out to Kombo support."""
    zohorecruit: NotRequired[PostAtsCandidatesRequestBodyZohorecruitTypedDict]
    r"""Fields specific to Zoho Recruit."""
    bullhorn: NotRequired[PostAtsCandidatesRequestBodyBullhornTypedDict]
    r"""Fields specific to Bullhorn."""
    smartrecruiters: NotRequired[PostAtsCandidatesRequestBodySmartrecruitersTypedDict]
    r"""Fields specific to SmartRecruiters."""
    talentadore: NotRequired[PostAtsCandidatesRequestBodyTalentadoreTypedDict]
    r"""Fields specific to Talentadore."""
    guidecom: NotRequired[PostAtsCandidatesRequestBodyGuidecomTypedDict]
    r"""Fields specific to GuideCom."""
    dvinci: NotRequired[PostAtsCandidatesRequestBodyDvinciTypedDict]
    r"""Fields specific to d.vinci."""
    hrworks: NotRequired[PostAtsCandidatesRequestBodyHrworksTypedDict]
    r"""Fields specific to GuideCom."""
    jobylon: NotRequired[PostAtsCandidatesRequestBodyJobylonTypedDict]
    r"""Fields specific to Jobylon."""
    avature: NotRequired[PostAtsCandidatesRequestBodyAvatureTypedDict]
    r"""Fields specific to Avature."""
    recruitee: NotRequired[PostAtsCandidatesRequestBodyRecruiteeTypedDict]
    r"""Fields specific to Recruitee."""
    rexx: NotRequired[PostAtsCandidatesRequestBodyRexxTypedDict]
    r"""Fields specific to Rexx."""
    umantis: NotRequired[PostAtsCandidatesRequestBodyUmantisTypedDict]
    r"""Fields specific to Abacus Umantis."""
    piloga: NotRequired[PostAtsCandidatesRequestBodyPilogaTypedDict]
    r"""Fields specific to P&I Loga."""
    pinpoint: NotRequired[PostAtsCandidatesRequestBodyPinpointTypedDict]
    r"""Fields specific to Pinpoint."""
    covetorest: NotRequired[PostAtsCandidatesRequestBodyCovetorestTypedDict]
    r"""Fields specific to Coveto REST."""


class PostAtsCandidatesRequestBodyRemoteFields(BaseModel):
    r"""Additional fields that we will pass through to specific ATS systems."""

    successfactors: Optional[PostAtsCandidatesRequestBodySuccessfactors] = None
    r"""Fields specific to SAP SuccessFactors."""

    personio: Optional[PostAtsCandidatesRequestBodyPersonio] = None
    r"""Fields specific to Personio."""

    talentsoft: Optional[PostAtsCandidatesRequestBodyTalentsoft] = None
    r"""Fields specific to TalentSoft."""

    teamtailor: Optional[PostAtsCandidatesRequestBodyTeamtailor] = None

    greenhouse: Optional[PostAtsCandidatesRequestBodyGreenhouse] = None
    r"""Fields specific to Greenhouse."""

    lever: Optional[PostAtsCandidatesRequestBodyLever] = None
    r"""Fields specific to Lever."""

    workable: Optional[PostAtsCandidatesRequestBodyWorkable] = None
    r"""Workable specific remote fields for ATS actions."""

    workday: Optional[PostAtsCandidatesRequestBodyWorkday] = None
    r"""Fields specific to Workday. The remote fields schema follows the documentation at https://community.workday.com/sites/default/files/file-hosting/productionapi/Recruiting/v43.0/Put_Candidate.html. Only defined fields are supported, if you need additional field support please reach out to Kombo support."""

    zohorecruit: Optional[PostAtsCandidatesRequestBodyZohorecruit] = None
    r"""Fields specific to Zoho Recruit."""

    bullhorn: Optional[PostAtsCandidatesRequestBodyBullhorn] = None
    r"""Fields specific to Bullhorn."""

    smartrecruiters: Optional[PostAtsCandidatesRequestBodySmartrecruiters] = None
    r"""Fields specific to SmartRecruiters."""

    talentadore: Optional[PostAtsCandidatesRequestBodyTalentadore] = None
    r"""Fields specific to Talentadore."""

    guidecom: Optional[PostAtsCandidatesRequestBodyGuidecom] = None
    r"""Fields specific to GuideCom."""

    dvinci: Optional[PostAtsCandidatesRequestBodyDvinci] = None
    r"""Fields specific to d.vinci."""

    hrworks: Optional[PostAtsCandidatesRequestBodyHrworks] = None
    r"""Fields specific to GuideCom."""

    jobylon: Optional[PostAtsCandidatesRequestBodyJobylon] = None
    r"""Fields specific to Jobylon."""

    avature: Optional[PostAtsCandidatesRequestBodyAvature] = None
    r"""Fields specific to Avature."""

    recruitee: Optional[PostAtsCandidatesRequestBodyRecruitee] = None
    r"""Fields specific to Recruitee."""

    rexx: Optional[PostAtsCandidatesRequestBodyRexx] = None
    r"""Fields specific to Rexx."""

    umantis: Optional[PostAtsCandidatesRequestBodyUmantis] = None
    r"""Fields specific to Abacus Umantis."""

    piloga: Optional[PostAtsCandidatesRequestBodyPiloga] = None
    r"""Fields specific to P&I Loga."""

    pinpoint: Optional[PostAtsCandidatesRequestBodyPinpoint] = None
    r"""Fields specific to Pinpoint."""

    covetorest: Optional[PostAtsCandidatesRequestBodyCovetorest] = None
    r"""Fields specific to Coveto REST."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "successfactors",
                "personio",
                "talentsoft",
                "teamtailor",
                "greenhouse",
                "lever",
                "workable",
                "workday",
                "zohorecruit",
                "bullhorn",
                "smartrecruiters",
                "talentadore",
                "guidecom",
                "dvinci",
                "hrworks",
                "jobylon",
                "avature",
                "recruitee",
                "rexx",
                "umantis",
                "piloga",
                "pinpoint",
                "covetorest",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostAtsCandidatesRequestBodyTypedDict(TypedDict):
    candidate: PostAtsCandidatesRequestBodyCandidateTypedDict
    application: PostAtsCandidatesRequestBodyApplicationTypedDict
    r"""Currently, every candidate has one application. If you are interested in talent pools, please contact Kombo."""
    screening_question_answers: NotRequired[
        List[PostAtsCandidatesRequestBodyScreeningQuestionAnswerTypedDict]
    ]
    r"""Array of answers to screening questions. Currently, not all question types are supported, and unsupported ones will not be submitted.

    The available questions for a job can be retrieved from the get jobs endpoint. The answers will be validated based on the format of the questions. Make sure to follow this schema to avoid errors.
    """
    attachments: NotRequired[List[PostAtsCandidatesRequestBodyAttachmentTypedDict]]
    r"""An array of the attachments you would like upload."""
    source: NotRequired[PostAtsCandidatesRequestBodySourceTypedDict]
    r"""**(⚠️ Deprecated - Use [automatic source writing](/ats/features/application-attribution#automatic-attribution) instead)** Optional source information that will be attached to the candidate. If
    you're a job board or recruiting service, you can use this to make sure your
    customers can see which candidates came from you.

    This is deprecated because writing sources requires users to do some setup in most ATSs.
    """
    sourced_by: NotRequired[PostAtsCandidatesRequestBodySourcedByTypedDict]
    r"""Credit the recruiter or team member who sourced this candidate.

    While the `source` field tracks the channel/platform (e.g., \"Awesome Jobboard\"), the `sourced_by` field tracks the individual person responsible for finding the candidate.
    """
    gdpr_consent: NotRequired[PostAtsCandidatesRequestBodyGdprConsentTypedDict]
    r"""Optional GDPR consent information required in some jurisdictions (like the Czech Republic or Slovakia)."""
    remote_fields: NotRequired[PostAtsCandidatesRequestBodyRemoteFieldsTypedDict]
    r"""Additional fields that we will pass through to specific ATS systems."""


class PostAtsCandidatesRequestBody(BaseModel):
    candidate: PostAtsCandidatesRequestBodyCandidate

    application: PostAtsCandidatesRequestBodyApplication
    r"""Currently, every candidate has one application. If you are interested in talent pools, please contact Kombo."""

    screening_question_answers: Optional[
        List[PostAtsCandidatesRequestBodyScreeningQuestionAnswer]
    ] = None
    r"""Array of answers to screening questions. Currently, not all question types are supported, and unsupported ones will not be submitted.

    The available questions for a job can be retrieved from the get jobs endpoint. The answers will be validated based on the format of the questions. Make sure to follow this schema to avoid errors.
    """

    attachments: Optional[List[PostAtsCandidatesRequestBodyAttachment]] = None
    r"""An array of the attachments you would like upload."""

    source: Annotated[
        Optional[PostAtsCandidatesRequestBodySource],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""**(⚠️ Deprecated - Use [automatic source writing](/ats/features/application-attribution#automatic-attribution) instead)** Optional source information that will be attached to the candidate. If
    you're a job board or recruiting service, you can use this to make sure your
    customers can see which candidates came from you.

    This is deprecated because writing sources requires users to do some setup in most ATSs.
    """

    sourced_by: Optional[PostAtsCandidatesRequestBodySourcedBy] = None
    r"""Credit the recruiter or team member who sourced this candidate.

    While the `source` field tracks the channel/platform (e.g., \"Awesome Jobboard\"), the `sourced_by` field tracks the individual person responsible for finding the candidate.
    """

    gdpr_consent: Optional[PostAtsCandidatesRequestBodyGdprConsent] = None
    r"""Optional GDPR consent information required in some jurisdictions (like the Czech Republic or Slovakia)."""

    remote_fields: Optional[PostAtsCandidatesRequestBodyRemoteFields] = None
    r"""Additional fields that we will pass through to specific ATS systems."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "screening_question_answers",
                "attachments",
                "source",
                "sourced_by",
                "gdpr_consent",
                "remote_fields",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PostAtsCandidatesRequestBodySuccessfactors.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyPostHeaders.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodySocialSuffixReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyNameDetailData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyGlobalPersonalInformationData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyJobAppliedToData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyFieldOfStudyReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyDegreeReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyEducationDatum.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodySkillDatum.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageDatumLanguageReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageProficiencyReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageAbilityTypeReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageAbilityData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageAbility.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguage.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLanguageDatum.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyExperienceDatum.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyResumeData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyJobApplicationData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyCountryRegionReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyCountryCityReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyLocationData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyContactData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyWorkerReference.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyCandidateData.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyWorkday.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyConsentDecisions.model_rebuild()
except NameError:
    pass
try:
    PostAtsCandidatesRequestBodyHrworks.model_rebuild()
except NameError:
    pass
